<?php

namespace Drupal\command_bus;

use Drupal\command_bus\Command\CommandHandlerInterface;
use Drupal\command_bus\Command\CommandInterface;

/**
 * Class CommandBus.
 *
 * @package Drupal\command_bus
 */
class CommandBus implements CommandBusInterface {

  /**
   * The command.
   *
   * @var \Drupal\command_bus\Command\CommandInterface
   */
  private $command;

  /**
   * The command handler.
   *
   * @var \Drupal\command_bus\Command\CommandHandlerInterface
   */
  private $commandHandler;

  /**
   * Resolves the command to a handler and handles the command handler.
   *
   * @param \Drupal\command_bus\Command\CommandInterface $command
   *   The command.
   */
  public function execute(CommandInterface $command) {
    $commandClass = get_class($command);
    $commandHandler = $this->resolve($commandClass);

    if ($commandHandler instanceof CommandHandlerInterface) {
      $this->command = $command;
      $this->commandHandler = $commandHandler;

      $this->handleCommand();
    }
  }

  /**
   * Resolves the command to a command handler.
   *
   * @param string $commandClass
   *   The command class string.
   *
   * @return bool|\Drupal\command_bus\Command\CommandHandlerInterface
   *   The command handler or FALSE if no valid handler is found.
   */
  private function resolve($commandClass) {
    $commandHandlerClass = $commandClass . 'Handler';

    if (class_exists($commandHandlerClass)) {
      return new $commandHandlerClass();
    }

    return FALSE;
  }

  /**
   * Handles the command.
   *
   * @return mixed
   *   The command handle result.
   */
  private function handleCommand() {
    $this->commandHandler->setCommand($this->command);

    return $this->commandHandler->handle();
  }

}
